class Config:
    Username = None
    User_id = None
    Search = None
    Geo = ""
    Location = False
    Near = None
    Lang = None
    Output = None
    Elasticsearch = None
    Year = None
    Since = None
    Until = None
    Email = False
    Phone = False
    Verified = False
    Store_csv = False
    Store_json = False
    Custom = {"tweet": None, "user": None, "username": None}
    Show_hashtags = False
    Show_cashtags = False
    Limit = None
    Count = None
    Stats = False
    Database = None
    To = None
    All = None
    Debug = False
    Format = None
    Essid = ""
    Profile = False
    Followers = False
    Following = False
    Favorites = False
    TwitterSearch = False
    User_full = False
    Profile_full = False
    Store_object = False
    Store_object_tweets_list = None
    Pandas_type = None
    Pandas = False
    Index_tweets = "twinttweets"
    Index_follow = "twintgraph"
    Index_users = "twintuser"
    Retries_count = 10
    Resume = None
    Images = False
    Videos = False
    Media = False
    Replies = False
    Pandas_clean = True
    Lowercase = True
    Pandas_au = True
    Proxy_host = ""
    Proxy_port = 0
    Proxy_type = None
    Tor_control_port = 9051
    Tor_control_password = None
    Retweets = False
    Query = None
    Hide_output = False
    Near = ""
    Custom_query = ""
    Popular_tweets = False
    Skip_certs = False
    Native_retweets = False
    Min_likes = 0
    Min_retweets = 0
    Min_replies = 0
    Links = None
    Source = None
    Members_list = None
    Filter_retweets = False
    Translate = False
    TranslateSrc = "en"
    TranslateDest = "en"    